package Blatt02_DiningSavages;

/**
 * Implementiert den Koch als eigenständigen Thread.
 * 
 * @author Sebastian Flothow (sebastian@flothow.de)
 */
public class Cook extends Thread {
	private Pot pot;
	private final static int SERVINGS_PER_BATCH = 5;

	/**
	 * Erzeugt einen Koch, der den angegebenen Topf fortlaufend füllt.
	 * 
	 * @param pot Topf, den der Koch füllen soll.
	 */
	public Cook(Pot pot) {
		this.pot = pot;
	}

	
	/**
	 * Füllt fortlaufend den Topf. Falls eine Exception auftritt wird diese auf
	 * stderr ausgegeben und der Thread beendet.
	 */
	@Override
	public void run() {
		try {
			while (true) {
				/* in this scenario we doesn't need a synchronized access for 
				 * the cook, because he will be informed only when the pot is 
				 * empty. 
				 * we have to lock to send and receive signals.
				 * now we wait until a savage cry for food.
				 * Only the savage cry for it, we fill the pot with food.
				 * when the pot is full, we inform the savage over that
				 * and unlock the lock
				 */
				this.pot.lock.lock();
				this.pot.potIsEmpty.await();
				pot.putServings(SERVINGS_PER_BATCH);
				System.out.println( "Koch füllt " 
						+ SERVINGS_PER_BATCH 
						+ " Portionen in den Topf");
				this.pot.potNotEmpty.signal();
				this.pot.lock.unlock();
			}
		} catch (Exception e) {
			System.err.print("Koch: ");
			e.printStackTrace();
		} finally { this.pot.lock.unlock(); }
	}
}
